<?php
/**
 *  产品分类
 *  @author  system
 *  @version    1.0
 *  @date 2018-07-20 11:16:57
 *
 */
namespace App\Http\Controllers\Admin\Furniture\Goods;
use App\Http\Controllers\Admin\Controller;
use App\Models\FurnitureGoodsCategoryModel;
use Illuminate\Http\Request;
use App\Repositories\Base\Criteria\OrderBy;
use App\Repositories\Furniture\Goods\Criteria\MultiWhere;
use App\Repositories\Furniture\Goods\CategoryRepository;

class CategoryController extends Controller
{
    private $repository;

    public function __construct(CategoryRepository $repository) {
        if(!$this->repository) $this->repository = $repository;
    }

    function index(Request $request) {
        $search['keyword'] = $request->input('keyword');
        $search['store_id'] = $this->getStoreId();
        $query = $this->repository->pushCriteria(new MultiWhere($search));

        if(isset($request['sort_field']) && $request['sort_field'] && isset($request['sort_field_by'])) {
        $query = $query->pushCriteria(new OrderBy($request['sort_field'],$request['sort_field_by']));
        }
        $list = $query->paginate(10);
        return view('admin.furniture.goods.category.index',compact('list'));
    }


    function check(Request $request) {
        $request = $request->all();
        $search['keyword'] = $request->input('keyword');
        $orderby = array();
        if(isset($request['sort_field']) && $request['sort_field'] && isset($request['sort_field_by'])) {
            $orderby[$request['sort_field']] = $request['sort_field_by'];
        }
        $list = $this->repository->search($search,$orderby);
        return view('admin.furniture.goods.category.check',compact('list'));
    }


    /**
     * 添加
     * 
     */
    public function create(Request $request)
    {
        if($request->method() == 'POST') {
            return $this->_createSave();
        }
        $cate = $this->cate(0);

        return view('admin.furniture.goods.category.edit',compact('cate'));
    }

    /**
     * 保存修改
     */
    private function _createSave(){
        $data = (array) request('data');
        $data['store_id'] =$this->getStoreId();

        $id = $this->repository->create($data);
        if($id) {
            $url[] = array('url'=>U( 'Furniture/Goods/Category/index'),'title'=>'返回列表');
            $url[] = array('url'=>U( 'Furniture/Goods/Category/create'),'title'=>'继续添加');
            $this->showMessage('添加成功',$url);
        }else{
            $url[] = array('url'=>U( 'Furniture/Goods/Category/index'),'title'=>'返回列表');
            return $this->showWarning('添加失败',$url);
        }
    }
    
    /**
     * 
     * 修改
     * 
     * 
     */
    public function update(Request $request) {
        if($request->method() == 'POST') {
            return $this->_updateSave();
        }
        $data = $this->repository->find($request->get('id'));
        $cate = $this->cate(0);
        return view('admin.furniture.goods.category.edit',compact('data','cate'));
    }

    /**
     * 保存修改
     */
    private function _updateSave() {
        $data = (array) request('data');
        $ok = $this->repository->update(request('id'),$data);
        if($ok) {
            $url[] = array('url'=>U( 'Furniture/Goods/Category/index'),'title'=>'返回列表');
            return $this->showMessage('操作成功',urldecode(request('_referer')));
        }else{
            $url[] = array('url'=>U( 'Furniture/Goods/Category/index'),'title'=>'返回列表');
            return $this->showWarning('操作失败',$url);
        }
    }

    public function view(Request $request) {
        $data = $this->repository->find(request('id'));
        return view('admin.furniture.goods.category.view',compact('data'));
    }


    /**
     *
     * 状态改变
     *
     */
    public function status(Request $request) {
        $ok = $this->repository->updateStatus(request('id'),request('status'));
        if($ok) {
            return $this->showMessage('操作成功');
        }else{
            return $this->showWarning('操作失败');
        }
    }
    
    /**
     * 删除
     */
    public function destroy(Request $request) {
        $bool = $this->repository->destroy($request->get('id'));
        if($bool) {
            return  $this->showMessage('操作成功');
        }else{
            return  $this->showWarning("操作失败");
        }
    }

    /*
         * 父分类的递归
         */
    public function cate($pid)
    {
        $res = FurnitureGoodsCategoryModel::where('pid', '=', $pid)
            ->where('status','1')
            ->where('store_id',$this->getStoreId())
            ->get();
        $data = [];
        foreach ($res as $key => $val) {
            $val->sub = self::cate($val->id);
            $data[] = $val;
        }
        return $data;
    }
}